﻿''' <summary>
''' 設定ファイルの管理
''' </summary>
''' <remarks></remarks>
Public Module Manager

    ''' <summary>システム設定ファイル名</summary>
    Public ReadOnly SystemSettingsFileName As String = IO.Path.GetFullPath(".\SystemSetting.xml")

    ''' <summary>鍵設定ファイル名</summary>
    Public ReadOnly keySettingsFileName As String = IO.Path.GetFullPath(".\KeySetting.xml")

    '起動中に何らかの理由によりカレントディレクトリが変更されてもいいように、起動時にパスを確定する。起動中に実行ファイルが移動（カット&ペーストやドラッグ等）してもカレントディレクトリは変更されないので、この1回で十分。
    'カレントディレクトリを使用することで、ショートカットアイコンの”作業フォルダ”で保存先を指定できるようになる。（ちなみに、《System.IO.Directory.GetCurrentDirectory()》でカレントディレクトリの設定・取得が可能。）
    'ファイル名の前の".\"は、別になくてもいいけどなんとなく付けてみた。ちなみに、"."は現在のフォルダ（即ちカレントフォルダ）、".."なら1つ上のフォルダを意味する。


    ''' <summary>システム設定パラメータ</summary>
    Public SystemSettings As New SystemParameter With {.Revision = SystemParameter.NOW_REVISION} '新規再生時のリビジョンを最新にするためにプロパティを設定する。

    ''' <summary>鍵設定パラメータ</summary>
    Public KeySettings As New KeyParameter With {.Revision = KeyParameter.NOW_REVISION} '新規再生時のリビジョンを最新にするためにプロパティを設定する。


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' 設定パラメータの復元
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="LoadObject">復元先</param>
    ''' <remarks></remarks>
    Private Sub SettingsLoadBase(FileName As String, ByRef LoadObject As Object)

        '=== XMLファイルから読み込む ===
        If IO.File.Exists(FileName) Then

            Try

                '--- XMLファイルから読み込み、逆シリアル化する ---
                Dim serializer As New Xml.Serialization.XmlSerializer(LoadObject.GetType) 'XmlSerializerオブジェクトの作成
                Using fs = New IO.FileStream(FileName, IO.FileMode.Open) 'ファイルを開く
                    LoadObject = CTypeDynamic(serializer.Deserialize(fs), LoadObject.GetType)
                End Using

            Catch ex As Exception

                Throw 'エラーを再スロー

            End Try

        End If

    End Sub

    ''' <summary>
    ''' システム設定パラメータの復元
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="LoadObject">復元先</param>
    ''' <returns>整合性エラーメッセージ</returns>
    ''' <remarks></remarks>
    Public Function SettingsLoad(FileName As String, ByRef LoadObject As SystemParameter) As String

        '=== 設定ファイルの読み込み ===
        Try
            SettingsLoadBase(FileName, LoadObject)
            SystemSettingsRevShifter() 'リビジョン移行
        Catch ex As Exception
            Throw
        End Try

        '=== 整合性エラーのチェック ===
        Return SystemParameterAdjustmentErrorCheck(LoadObject)

    End Function

    ''' <summary>
    ''' 鍵設定パラメータの復元
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="LoadObject">復元先</param>
    ''' <remarks></remarks>
    Public Sub SettingsLoad(FileName As String, ByRef LoadObject As KeyParameter)

        '=== 設定ファイルの読み込み ===
        Try
            SettingsLoadBase(FileName, LoadObject)
            KeySettingsRevShifter() 'リビジョン移行
        Catch ex As Exception
            Throw
        End Try

    End Sub

    ''' <summary>
    ''' 設定パラメータの保存
    ''' </summary>
    ''' <param name="FileName">ファイル名</param>
    ''' <param name="SaveObject">保存元</param>
    ''' <param name="HideMode">不可視属性モード（《Ture》 = 不可視化 / 《False》 = 現状維持）</param>
    ''' <param name="BackupExtension">バックアップ用拡張子</param>
    ''' <remarks></remarks>
    Public Sub SettingsSave(FileName As String, SaveObject As Object, Optional HideMode As Boolean = False, Optional BackupExtension As String = "bak")

        '=== XMLファイルに書き込む ===
        Try

            If IO.File.Exists(FileName) Then
                '-- 旧ファイルのバックアップ ---
                IO.File.Copy(FileName, IO.Path.ChangeExtension(FileName, BackupExtension), True)

                '---不可視属性の解除（しないと書き込めない） ---
                Dim fi As New IO.FileInfo(FileName)
                If (fi.Attributes And IO.FileAttributes.Hidden) = IO.FileAttributes.Hidden Then
                    fi.Attributes = fi.Attributes And Not IO.FileAttributes.Hidden
                    HideMode = True
                End If
            End If

            '--- シリアル化し、XMLファイルに保存する ---
            Dim serializer As New Xml.Serialization.XmlSerializer(SaveObject.GetType) 'XmlSerializerオブジェクトを作成
            Using fs = New IO.FileStream(FileName, IO.FileMode.Create) 'ファイルを開く
                serializer.Serialize(fs, SaveObject)
            End Using

            '---不可視属性の設定・復元 ---
            If HideMode Then
                Dim fi As New IO.FileInfo(FileName) '解除時のがあれば再インスタンシング不要かもしれないけど、ファイルが置き換わってるし念のためやっとく。使いまわしても、実質的に問題なさそうやけど。
                fi.Attributes = fi.Attributes Or IO.FileAttributes.Hidden
            End If

        Catch ex As Exception

            Throw 'エラーを再スロー

        End Try

    End Sub

    ''' <summary>
    ''' システム設定パラメータの整合性確認と補正を行う。
    ''' </summary>
    ''' <param name="value">対象の値</param>
    ''' <returns>整合性エラーメッセージ</returns>
    Private Function SystemParameterAdjustmentErrorCheck(value As SystemParameter)

        Dim errorPoint As New Text.StringBuilder

        With value

            With .TaskTray
                With .Icon
                    .Title = .Title.Substring(0, Math.Min(.Title.Length, SystemParameter.TaskTrayParameter.IconParameter.TitleMaxLength)) '表題（63文字以下）
                    '.Title = Left(.Title, SystemParameter.TaskTrayParameter.IconParameter.TitleMaxLength) '上と同じ。こっちの方が文字長例外が無くて簡潔だけどVB方言なのが玉に瑕。
                End With
                With .HoverPopupper
                    .AdjustStep = Math.Max(50, Math.Min(.AdjustStep, 1000)) '調整間隔（50 <= x <=1000）
                    .Time = Math.Max(0, Math.Min(.Time, SystemParameter.TaskTrayParameter.TimeMeasureParameter.AdjustMax)) '時間（0 <= x <= AdjustMax）
                    '.Time = Math.Round(.Time / .AdjustStep, MidpointRounding.AwayFromZero) * .AdjustStep '切りのいい数値に整える。
                End With
                With .AutoCloser
                    .AdjustStep = Math.Max(50, Math.Min(.AdjustStep, 1000)) '調整間隔（50 <= x <=1000）
                    .Time = Math.Max(0, Math.Min(.Time, SystemParameter.TaskTrayParameter.TimeMeasureParameter.AdjustMax)) '時間（0 <= x <= AdjustMax）
                    '.Time = Math.Round(.Time / .AdjustStep, MidpointRounding.AwayFromZero) * .AdjustStep '切りのいい数値に整える。
                End With
            End With

            With .LaunchMenu
                .IconImageScale = Math.Min(Math.Max(.IconImageScale, SystemParameter.LaunchMenuParameter.SmallIconScale), SystemParameter.LaunchMenuParameter.MaximumIconScale)
                With .MenuBuild
                    If .RecursiveCallMax < 0 Then errorPoint.AppendLine("  <ランチメニュー>-<再帰呼出限界深度>-<再帰呼出限界深度> ⇒ 0以上")
                    .Depth = Math.Min(Math.Max(.Depth, 0), .RecursiveCallMax) '一括構築深度（0 <= x <= RecursiveCallMax）
                End With
                With .Popup
                    If .PopupCheckTime < 100 Then errorPoint.AppendLine("  <ランチメニュー>-<展開>-<成否確認時間> ⇒ 100以上")
                End With
            End With

            With .SecretMenu.MouseWheel
                If .RollSeparateTime < 0 Then errorPoint.AppendLine("  <裏メニュー>-<マウスホイール>-<回転分離閾時間> ⇒ 0以上")
            End With

        End With

        Return errorPoint.ToString

    End Function

    ''' <summary>
    ''' システム設定のリビジョンを移行する
    ''' </summary>
    Private Sub SystemSettingsRevShifter()

        '=== 移行処理 ===
        With SystemSettings

            Dim br As Integer = .Revision '開始時のリビジョン

            '--- Rev.0 から Rev.1 へ ---
            If .Revision = 0 Then

                Dim xd = New Xml.XmlDocument
                xd.Load(SystemSettingsFileName)

                'ゲージ更新間隔を初期化（ゲージ増量（棒から円へ）でスムーズさが重要になったので）
                With .TaskTray
                    .HoverPopupper.AdjustStep = 50
                    .AutoCloser.AdjustStep = 50
                End With

                '・アイコン画像の大きさ（フラグからピクセル数に変えたので）
                Dim v As Boolean
                If Boolean.TryParse(xd.SelectSingleNode("//大アイコン").InnerText, v) Then
                    SystemSettings.LaunchMenu.IconImageScale = If(v, SystemParameter.LaunchMenuParameter.LargeIconScale, SystemParameter.LaunchMenuParameter.SmallIconScale)
                End If

                .Revision = 1

            End If

            '--- Rev.1 から Rev.2 へ ---
            'If .Revision = 1 Then
            '   '～ 何らかの移行処理 ～
            '    .Revision = 2
            'End If

            '--- Rev.2 から Rev.3 へ ---
            'If .Revision = 2 Then
            '   '～ 何らかの移行処理 ～
            '    .Revision = 3
            'End If

            '…と続けていく。

            '--- 保存 ---
            If .Revision > br Then
                SettingsSave(SystemSettingsFileName, SystemSettings,, "old")
            End If

        End With

    End Sub

    ''' <summary>
    ''' 鍵設定のリビジョンを移行する
    ''' </summary>
    Private Sub KeySettingsRevShifter()

        '=== 移行処理 ===
        With KeySettings

            Dim br As Integer = .Revision '開始時のリビジョン

            '--- Rev.0 から Rev.1 へ ---
            If .Revision = 0 Then
                '・リビジョン属性を追記する。
                .Revision = 1
            End If

            '--- Rev.1 から Rev.2 へ ---
            'If .Revision = 1 Then
            '   '～ 何らかの移行処理 ～
            '    .Revision = 2
            'End If

            '--- Rev.2 から Rev.3 へ ---
            'If .Revision = 2 Then
            '   '～ 何らかの移行処理 ～
            '    .Revision = 3
            'End If

            '…と続けていく。

            '--- 保存 ---
            If .Revision > br Then
                SettingsSave(keySettingsFileName, KeySettings,, "old")
            End If

        End With

    End Sub

End Module
